static char *sccsid = "mqsend.c,  Version 1 Release 5, 23/05/2001";
#define TITLE "MQSeries Services send program for Expedite"
/******************************************************************/
/*                                                                */
/* (c) Copyright IBM Corp. 2000  All rights reserved.             */
/*                                                                */
/* This sample program is owned by International Business Machines*/
/* Corporation or one of its subsidiaries ("IBM") and is          */
/* copyrighted and licensed, not sold.                            */
/*                                                                */
/* You may copy, modify, and distribute this sample program in    */
/* any form without payment to IBM,  for any purpose including    */
/* developing, using, marketing or distributing programs that     */
/* include or are derivative works of the sample program.         */
/*                                                                */
/* The sample program is provided to you on an "AS IS" basis,     */
/* without warranty of any kind. IBM HEREBY  EXPRESSLY DISCLAIMS  */
/* ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT  */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND      */
/* FITNESS FOR A PARTICULAR PURPOSE.                              */
/* Some jurisdictions do not allow for the exclusion or limitation*/
/* of implied warranties, so the above limitations or exclusions  */
/* may not apply to you. IBM shall not be liable for any damages  */
/* you suffer as a result of using, modifying or distributing the */
/* sample program or its derivatives.                             */
/*                                                                */
/* Each copy of any portion of this sample program or any         */
/* derivative work, must include the above copyright notice and   */
/* disclaimer of warranty.                                        */
/*                                                                */
/******************************************************************/
/*                                                                */
/* Program name: mqsend                                           */
/*                                                                */
/* Description: Send files using Expedite profiles                */
/*                                                                */
/******************************************************************/
/*                                                                */
/* Version 1.0, 03/06/2000, First release NT                      */
/* Version 1.1, 21/07/2000, FindKeyWord added                     */
/* Version 1.2, 31/07/2000, Support for MQSIE header added        */
/* Version 1.3, 02/08/2000, First release AIX                     */
/* Version 1.4, 22/08/2000, Cleanup for SupportPac release        */
/* Version 1.5, 23/05/2001, First release OS/400                  */
/*                                                                */
/******************************************************************/
#include <stdio.h>                    /* ANSI                     */
#include <stdlib.h>                   /* ANSI                     */
#include <string.h>                   /* ANSI                     */
#include <time.h>                     /* ANSI                     */
/* MQSeries product include file                                  */
#include <cmqc.h>
/* Utils include files                                            */
#include "../../../src/file_util.h"   /* file functions           */
#include "../../../src/mq_util.h"     /* MQSeries functions       */
#include "../../../src/str_util.h"    /* string functions         */
/* MQSeries Services include files                                */
#include "../src/getmailboxindex.h"
#include "../src/keyword.h"
/* Expedite specific include file                                 */
#include "expedite.h"
/* Program specific include files                                 */
#include "profile.h"
#include "mqsend.h"

int main(int argc, char *argv[])
{
   char ProfileName[PATH_MAX];
   /* Expedite files variables                                    */
   char BaseInFileName[PATH_MAX];
   char BaseOutFileName[PATH_MAX];

   char LogText[1024];

   PINITFILE   InitFile = NULL;
   PIESENDINFO SendInfo = NULL;

   /* MQSeries variables                                          */
   MQHCONN  Hcon = MQHC_UNUSABLE_HCONN;/* connection handle       */
   MQLONG   Reason;                    /* reason code             */

   char LineBuffer[255];

   FILE *pBaseIn  = NULL;
   FILE *pBaseOut = NULL;

   MQSIE     Mqsie;
   char      IncludeMqsieHeader = FALSE;
   char     *MqsieHeader = NULL;

   int LineCount = 0;
   int i         = 0;

   strncpy(ProfileName, argc == 1 ? "profile" : argv[1], PATH_MAX);

   if ( (InitFile = ReadProfile(ProfileName)) == NULL )
      return(-1);
   
   if (LogInitFile(InitFile, TITLE) != 0 ) {
      free(InitFile);
      return(-1);
   }

   if ( (Reason = GetExpediteBaseFiles(InitFile, BaseInFileName, &pBaseIn, BaseOutFileName, &pBaseOut)) != 0) {
      free(InitFile);
      return Reason;
   }

   if ( (SendInfo = (PIESENDINFO)malloc(sizeof(IESENDINFO))) == NULL) {
      fprintf(stderr, "Cannot allocate memory !\n");
      free(InitFile);
      return(-1);
   }

   if ( (Reason = MQconn(InitFile->logdir, InitFile->qmgr, &Hcon) ) != 0 ) {
      free(InitFile);
      free(SendInfo);
      return Reason;
   }

   LineCount = 0;

   while ( fgets(LineBuffer, sizeof(LineBuffer) - 1, pBaseIn) ) {
      LineCount++;
      /* copy entry into baseout.msg */
      fputs(LineBuffer,pBaseOut);
      fflush(pBaseOut);
      LineBuffer[strlen(LineBuffer) - 1] = '\0';

      if (LineBuffer[0] == '#') {
         sprintf(LogText,"Ignoring line #%d, # at the beginning", LineCount);
         WriteLogFile(InitFile->logdir, LOG_INFO, LogText);
         continue;
      } /* if (LineBuffer[0] == '#') */

      if(FindKeyWord(LineBuffer, strlen(LineBuffer), "SEND") != NULL) {
         GetSendInfo(SendInfo, LineBuffer, strlen(LineBuffer));

         sprintf(LogText,"Processing line #%d", LineCount);
         WriteLogFile(InitFile->logdir, LOG_INFO, LogText);

         if(SendInfo->Class[0] != '\0') {
            WriteLogFile(InitFile->logdir, LOG_INFO, "CLASS found in this SEND entry");
            SetMQSIE(&Mqsie, "CLASS", SendInfo->Class);
            IncludeMqsieHeader = TRUE;
            MqsieHeader = CreateMQSIEheader(&Mqsie);
         } /* if(SendInfo->Class[0] != '\0') */
         else
            WriteLogFile(InitFile->logdir, LOG_WARNING, "CLASS not found in this SEND entry");

         if ((Reason = SendFile( InitFile->logdir,
                                 Hcon, 
                                 SendInfo->DestinationQ,
                                 "MQ31",
                                 InitFile->replyq,
                                 InitFile->replyqmgr,
                                 IncludeMqsieHeader == TRUE ? MQFMT_MQSIE : MQFMT_STRING,
                                 SendInfo->FileId,
                                 IncludeMqsieHeader == TRUE ? MqsieHeader : NULL,
                                 IncludeMqsieHeader == TRUE ? strlen(MqsieHeader) : 0,
                                 NULL,
                                 0 )) != 0) {

            printf("Cannot send file %s\n", SendInfo->FileId);
            /* create entry in baseout.msg */
            sprintf(LineBuffer, "NOT SENT;\nRETURN(0%d);\n", Reason);
            fputs(LineBuffer,pBaseOut);
            fflush(pBaseOut);
         }
         else {
            sprintf(LogText, "Sent file <%s> to Q <%s> with%s MQSIE header", SendInfo->FileId, SendInfo->DestinationQ, IncludeMqsieHeader == TRUE ? "" : "out");
			   WriteLogFile(InitFile->logdir, LOG_INFO, LogText);
            /* create entry in baseout.msg */ 			
            sprintf(LineBuffer, "SENT LENGTH(%ld);\nRETURN(00000);\n", SendInfo->FileSize);
            fputs(LineBuffer,pBaseOut);
            fflush(pBaseOut);
            if(IncludeMqsieHeader)
               free(MqsieHeader);
         }
         
      } /* (FindKeyWord(LineBuffer, "SEND") != NULL) */
      else{
         sprintf(LogText,"Ignoring line #%d, no SEND keyword not found", LineCount);
         WriteLogFile(InitFile->logdir, LOG_INFO, LogText);
      }
      
   } /*    while ( fgets(buffer, sizeof(buffer) - 1, fp) ) */

   MQdisc(InitFile->logdir, &Hcon);

   free(InitFile);
   free(SendInfo);

   return 0;
}
