#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
/* MQSeries product include file                                  */
#include <cmqc.h>
/* Utils include files                                            */
#include "../../../src/defs.h"        /* general include file     */
#include "../../../src/file_util.h"   /* file functions           */
#include "../../../src/mq_util.h"     /* MQSeries functions       */
#include "../../../src/str_util.h"    /* string functions         */
/* MQSeries Services include files                                */
#include "../src/mqsie.h"
/* Functions specific include file                                */
#include "getmailboxindex.h"

#define BUFFERLENGTH 1024

PMQMSGINFO *GetMailBoxIndex(char *LogDir, MQHCONN  Hconn, MQCHAR48 QName, MQLONG *Qdepth)
{
   int i = 0;

   PMQMSGINFO *RecvMsgInfo = NULL;

   MQLONG  CompCode	= MQCC_UNKNOWN;  /* completion code             */
   MQLONG  Reason		= MQRC_NONE;     /* reason code                 */
   /* variables for MQOPEN */
   MQOD    ObjDesc		= { MQOD_DEFAULT };
   MQLONG  OpenOptions	= 0;
   MQHOBJ  Hobj			= 0;
   /* variables for MQGET */
   MQGMO   GetMsgOpts	= { MQGMO_DEFAULT };
   MQMD    MsgDesc		= { MQMD_DEFAULT };
   PMQMD   pmdin			= NULL;
   PMQGMO  pgmoin			= NULL;
   PMQBYTE Buffer			= NULL;
   MQLONG  BufferLength = BUFFERLENGTH;
   MQLONG  DataLength	= 0;

   if ( (Reason = GetQueueDepth(LogDir, Hconn, QName, Qdepth) != (MQLONG)0 ) )
      return NULL;

   if (*Qdepth == 0)
      return NULL;

   MsgDesc.Version = MQMD_VERSION_2 ;

   if ((pmdin  = malloc(sizeof(MQMD))) == NULL)
		return NULL;																				/* 20/08/2001 */
   if ((pgmoin = malloc(sizeof(MQGMO))) == NULL) {
		free(pmdin);																				/* 20/08/2001 */
		return NULL;																				/* 20/08/2001 */
	}
   if ((Buffer = malloc(BUFFERLENGTH)) == NULL) {
		free(pmdin);																				/* 20/08/2001 */
		free(pgmoin);																				/* 20/08/2001 */
		return NULL;																				/* 20/08/2001 */
	}																									/* 20/08/2001 */

   OpenOptions = MQOO_BROWSE;

   strncpy(ObjDesc.ObjectName, QName, (size_t)MQ_Q_NAME_LENGTH);

   MQOPEN(Hconn, &ObjDesc, OpenOptions, &Hobj, &CompCode, &Reason);
   (void)WriteLogMQ(LogDir, _MQOPEN, CompCode, Reason);
	/* Return value (type int) ignored */

   GetMsgOpts.Version = MQGMO_VERSION_2;

   GetMsgOpts.MatchOptions = MQMO_NONE;

   GetMsgOpts.Options  = MQGMO_NO_WAIT ;
   GetMsgOpts.Options += MQGMO_BROWSE_NEXT ;
   GetMsgOpts.Options += MQGMO_ACCEPT_TRUNCATED_MSG;
   
   pgmoin = memcpy(pgmoin, &GetMsgOpts, sizeof(MQGMO));
   pmdin =  memcpy(pmdin,  &MsgDesc,	 sizeof(MQMD));
   
   i = 0;

   RecvMsgInfo = (PMQMSGINFO *)malloc(sizeof(PMQMSGINFO) * (*Qdepth));
	if (RecvMsgInfo == NULL) {																	/* 20/08/2001 */
		free(pmdin);																				/* 20/08/2001 */
		free(pgmoin);																				/* 20/08/2001 */
		free(Buffer);																				/* 20/08/2001 */
		return NULL;																				/* 20/08/2001 */
	}																									/* 20/08/2001 */

   /* Loop until MQGET unsuccessful             */
   while((i < (int)(*Qdepth)) && (CompCode == MQCC_OK)) {
      RecvMsgInfo[i] = (PMQMSGINFO)malloc(sizeof(MQMSGINFO));
		if (RecvMsgInfo[i] == NULL) {
			free(pmdin);																			/* 20/08/2001 */
			free(pgmoin);																			/* 20/08/2001 */
			free(Buffer);																			/* 20/08/2001 */
			free(RecvMsgInfo);																	/* 20/08/2001 */
			return NULL;																			/* 20/08/2001 */
		}

      /* Initialize the buffer to blanks               */
      memset(Buffer,(int)' ',BUFFERLENGTH);

      MQGET( Hconn,
             Hobj,
             pmdin,
             pgmoin,
             BufferLength,
             Buffer,
             &DataLength,
             &CompCode,
             &Reason );

      if  (CompCode != MQCC_OK)
         (void)WriteLogMQ(LogDir, _MQGET, CompCode, Reason);
			/* Return value (type int) ignored */
      else {
         memcpy(RecvMsgInfo[i]->CorrelId, pmdin->CorrelId, sizeof(RecvMsgInfo[i]->CorrelId));
         memcpy(RecvMsgInfo[i]->Format,   pmdin->Format,   sizeof(RecvMsgInfo[i]->Format));
         memcpy(RecvMsgInfo[i]->MsgId,    pmdin->MsgId,    sizeof(RecvMsgInfo[i]->MsgId));

         memcpy(RecvMsgInfo[i]->ReplyToQ, pmdin->ReplyToQ, sizeof(RecvMsgInfo[i]->ReplyToQ));
         StrTrimI(RecvMsgInfo[i]->ReplyToQ, (int)MQ_Q_NAME_LENGTH);

         RecvMsgInfo[i]->DataLength = DataLength;

         GetMQDate(RecvMsgInfo[i]->Date, pmdin);
         GetMQTime(RecvMsgInfo[i]->Time, pmdin);

         if(EQUALM(pmdin->Format, MQFMT_MQSIE, sizeof(pmdin->Format)))
            GetMQSIE(&RecvMsgInfo[i]->Mqsie, Buffer, DataLength);

         i = i + 1;
      }
  
 	}

	MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);										/* 19/08/2001 */
   (void)WriteLogMQ(LogDir, _MQCLOSE, CompCode, Reason);
	/* Return value (type int) ignored */

   free(pmdin);
   free(pgmoin);
   free(Buffer);

   return(RecvMsgInfo);
}


void DisplayMailBoxIndex(char *LogDir, PMQMSGINFO *RecvMsgInfo, char *DisMqsie, char *LogMqsie, MQLONG Qdepth)
{
   MQLONG i = 0;
   static char LogText[1024];

   while(i < Qdepth) {
      sprintf(LogText, "Msg #%ld from %.*s, %ld bytes [%.8s], %.*s, %.*s",
                 i + 1,
                 (int)MQ_Q_NAME_LENGTH, RecvMsgInfo[i]->ReplyToQ, 
                 RecvMsgInfo[i]->DataLength, 
                 RecvMsgInfo[i]->Format,
                 MSGDATELEN, RecvMsgInfo[i]->Date,
                 MSGTIMELEN, RecvMsgInfo[i]->Time );

#ifdef _DEBUG
      *DisMqsie = 'Y';
#endif

      if(*DisMqsie == 'Y')
         printf("%s\n", LogText);

      if(*LogMqsie == 'Y')
         (void)WriteLogFile(LogDir, LOG, LogText);
			/* Return value (type int) ignored */

      if(EQUALM(RecvMsgInfo[i]->Format, MQFMT_MQSIE, sizeof(RecvMsgInfo[i]->Format))) {
         if(*LogMqsie == 'Y')
            LogMQSIE(LogDir, &RecvMsgInfo[i]->Mqsie);

         if(*DisMqsie == 'Y')
            DisplayMQSIE(&RecvMsgInfo[i]->Mqsie);
      } /* if(EQUALM(RecvMsgInfo[i]->Format, MQFMT_MQSIE, sizeof(RecvMsgInfo[i]->Format))) */

      i = i + 1;

   } /* while(i < Qdepth) */
}


void FreeMQMSGINFO(PMQMSGINFO *RecvMsgInfo, MQLONG Qdepth)
{
   MQLONG i = 0;

   for (i = 0; i < Qdepth; i++)
      free(RecvMsgInfo[i]);

   free(RecvMsgInfo);
}
