#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
/* Utils include files                                            */
#include "defs.h"                     /* general include file     */
/* Functions specific include file                                */
#include "file_util.h"

int ExistFile(char *FileName)
{
   FILE *File = NULL;

   if ( ( File = fopen(FileName,"r")) == NULL )
      return(FALSE);
   
   return(fclose(File));
}


char *CheckPath(char *FullPath)
{
   int Length = (int)strlen(FullPath);

   if ( FullPath[Length - 1] != SEPARATOR ) {
      FullPath[Length] = SEPARATOR;
      FullPath[Length + 1] = '\0';
   }

	return(FullPath);
}


char *AddPath(char *Out, const char *Path, const char *FileName)
{
   strncpy(Out, Path, PATH_MAX);
   strncat(Out, FileName, PATH_MAX);

   return(Out);
}


long _FileSize(char *FileName)
{
   FILE *File  = NULL;
   long Length = 0;

   if ( ( File = fopen(FileName, "r") ) == NULL )
      return(-1);

   if (fseek(File, 0L, SEEK_END) != 0)
      return(-1);

   Length = ftell(File);

   if ( fclose(File) == EOF) {
      fprintf(stderr, "Error closing file %s, function = FileSize", FileName);
      return(-1);
   }

   return(Length);
}


FILE *FileOpen(char *LogDir, char *FileName, char *FileAttribute)
{
   FILE *File = NULL;
   char LogText[255];

   if ( ( File = fopen(FileName, FileAttribute) ) == NULL ) {
      sprintf(LogText, "Error open file %s, attribute = %s", FileName, FileAttribute);
      (void)WriteLogFile(LogDir, (char)LOG, LogText);
      return(NULL);
   }
   else
      return(File);
}


int FileClose(char *LogDir, FILE *File, char *FileName)
{
   char LogText[255];

   if ( fclose(File) == EOF) {
      sprintf(LogText, "Error closing file %s", FileName);
      (void)WriteLogFile(LogDir, (char)LOG, LogText);
      return(1);
   }
   else
      return(0);
}


#ifdef UTILV1
int WriteLogFile(char *LogDir, char ErrorNoError, char *LogText)
{
   struct tm   *time_now = NULL;
   FILE        *LogFile  = NULL;
   time_t      secs_now;
   char        TimeString[MAXTIMESIZE];
   char        FullPath[PATH_MAX];

   strcpy(FullPath, LogDir);

   if (ErrorNoError == (char)LOGERR)
     strcat(FullPath, "LOGERR");
   else
     strcat(FullPath, "LOG");


   if (_FileSize(FullPath) > MAXLOGSIZE) {
      if (CycleLogFile(FullPath) != 0) {
         fprintf(stderr, "Error cycle logfile ! %s\n", FullPath);
         return(4);
      }
   }

   if ( ( LogFile = fopen(FullPath, "a") ) == NULL ) {
      fprintf(stderr, "Error opening logfile! %s\n", FullPath);
      return(1);
   }

   (void)time(&secs_now);
   time_now = localtime(&secs_now);
   strftime(TimeString, MAXTIMESIZE, "%Y/%m/%d - %H:%M:%S", time_now);

   fprintf(LogFile, "%s - %s\n", TimeString, LogText);
   (void)fflush(LogFile);

   if (fclose(LogFile) == EOF) {
      fprintf(stderr, "Error closing logfile! %s\n", FullPath);
      return(2);
   }

#ifdef _DEBUG
   printf("%s\n", LogText);
#endif

   return(0);
}
#else
int WriteLogFile(char *LogDir, char Class, char *LogText)
{
   struct tm   *time_now = NULL;
   FILE        *LogFile  = NULL;
   time_t      secs_now;
   char        TimeString[MAXTIMESIZE];
   char        FullPath[PATH_MAX];

	static char *strClass[] = { "INFO", "WARNING", "ERROR" };

	if (Class < 0 || Class > 2)
		Class = 0;

   strcpy(FullPath, LogDir);
	strcat(FullPath, "MQILOG");

   if (_FileSize(FullPath) > MAXLOGSIZE) {
      if (CycleLogFile(FullPath) != 0) {
         fprintf(stderr, "Error cycle logfile ! %s\n", FullPath);
         return(4);
      }
   }

   if ( ( LogFile = fopen(FullPath, "a") ) == NULL ) {
      fprintf(stderr, "Error opening logfile! %s\n", FullPath);
      return(1);
   }

   (void)time(&secs_now);
   time_now = localtime(&secs_now);
   strftime(TimeString, MAXTIMESIZE, "%Y/%m/%d - %H:%M:%S", time_now);

   fprintf(LogFile, "%s - %s - %s\n", TimeString, strClass[Class], LogText);
   (void)fflush(LogFile);

   if (fclose(LogFile) == EOF) {
      fprintf(stderr, "Error closing logfile! %s\n", FullPath);
      return(2);
   }

#ifdef _DEBUG
   printf("%s\n", LogText);
#endif

   return(0);
}
#endif

int CycleLogFile(char *FullPath)
{
   int ReturnValue = -1;

   char FullPath1[PATH_MAX];
   char FullPath2[PATH_MAX];

   strcpy(FullPath1,FullPath);
   strcat(FullPath1,".1");

   if (ExistFile(FullPath1)) {
      strcpy(FullPath2,FullPath);
      strcat(FullPath2,".2");

      if (ExistFile(FullPath2) == TRUE) {
         if ((ReturnValue = remove(FullPath2)) != 0)
            return(ReturnValue);
      }

      if ((ReturnValue = rename(FullPath1, FullPath2)) != 0)
         return(ReturnValue);
   }

   if ((ReturnValue = remove(FullPath1)) != 0)
      return(ReturnValue);

   return(rename(FullPath, FullPath1));
}



